using System;
using Server;
using Server.Menus;
using Server.Menus.Questions;
using Server.Spells;
using Server.Targeting;

namespace Server.Scripts.Commands
{
	public class CommandHandlers
	{
		public static void Initialize()
		{
			Register( "Delete", AccessLevel.GameMaster, new CommandEventHandler( Delete_OnCommand ) );
			Register( "Remove", AccessLevel.GameMaster, new CommandEventHandler( Delete_OnCommand ) );

			Register( "ShaveHair", AccessLevel.GameMaster, new CommandEventHandler( ShaveHair_OnCommand ) );
			Register( "ShaveBeard", AccessLevel.GameMaster, new CommandEventHandler( ShaveBeard_OnCommand ) );

			Register( "Kill", AccessLevel.GameMaster, new CommandEventHandler( Kill_OnCommand ) );

			Register( "Resurrect", AccessLevel.GameMaster, new CommandEventHandler( Resurrect_OnCommand ) );
			Register( "Res", AccessLevel.GameMaster, new CommandEventHandler( Resurrect_OnCommand ) );

			Register( "NewXYZ", AccessLevel.GameMaster, new CommandEventHandler( NewXYZ_OnCommand ) );
			Register( "NewX", AccessLevel.GameMaster, new CommandEventHandler( NewX_OnCommand ) );
			Register( "NewY", AccessLevel.GameMaster, new CommandEventHandler( NewY_OnCommand ) );
			Register( "NewZ", AccessLevel.GameMaster, new CommandEventHandler( NewZ_OnCommand ) );

			Register( "IncXYZ", AccessLevel.GameMaster, new CommandEventHandler( IncXYZ_OnCommand ) );
			Register( "IncX", AccessLevel.GameMaster, new CommandEventHandler( IncX_OnCommand ) );
			Register( "IncY", AccessLevel.GameMaster, new CommandEventHandler( IncY_OnCommand ) );
			Register( "IncZ", AccessLevel.GameMaster, new CommandEventHandler( IncZ_OnCommand ) );

			Register( "AutoPageNotify", AccessLevel.Counselor, new CommandEventHandler( APN_OnCommand ) );
			Register( "APN", AccessLevel.Counselor, new CommandEventHandler( APN_OnCommand ) );

			Register( "Animate", AccessLevel.Counselor, new CommandEventHandler( Animate_OnCommand ) );

			Register( "Tele", AccessLevel.Counselor, new CommandEventHandler( Teleport_OnCommand ) );
			Register( "Teleport", AccessLevel.Counselor, new CommandEventHandler( Teleport_OnCommand ) );

			Register( "Hide", AccessLevel.Counselor, new CommandEventHandler( Hide_OnCommand ) );
			Register( "Unhide", AccessLevel.Counselor, new CommandEventHandler( Unhide_OnCommand ) );

			Register( "Cast", AccessLevel.Counselor, new CommandEventHandler( Cast_OnCommand ) );

			Register( "Stuck", AccessLevel.Counselor, new CommandEventHandler( Stuck_OnCommand ) );
		}

		public static void Register( string command, AccessLevel access, CommandEventHandler handler )
		{
			Server.Commands.Register( command, access, handler );
		}

		private class DeleteTarget : Target
		{
			public DeleteTarget() : base( -1, false, TargetFlags.None )
			{
			}

			protected override void OnTarget( Mobile from, object targeted )
			{
				if ( targeted is Item )
				{
					((Item)targeted).Delete();
				}
				else if ( targeted is Mobile && !((Mobile)targeted).Player )
				{
					((Mobile)targeted).Delete();
				}
				else
				{
					from.SendMessage( "That cannot be deleted." );
				}
			}
		}

		public static void Delete_OnCommand( CommandEventArgs e )
		{
			e.Mobile.Target = new DeleteTarget();
		}

		private class DeleteItemByLayerTarget : Target
		{
			private Layer m_Layer;

			public DeleteItemByLayerTarget( Layer layer ) : base( -1, false, TargetFlags.None )
			{
				m_Layer = layer;
			}

			protected override void OnTarget( Mobile from, object targeted )
			{
				if ( targeted is Mobile )
				{
					Item item = ((Mobile)targeted).FindItemOnLayer( m_Layer );

					if ( item != null )
						item.Delete();
				}
				else
				{
					from.SendMessage( "Target a mobile." );
				}
			}
		}

		private static void ShaveHair_OnCommand( CommandEventArgs e )
		{
			e.Mobile.Target = new DeleteItemByLayerTarget( Layer.Hair );
		}

		private static void ShaveBeard_OnCommand( CommandEventArgs e )
		{
			e.Mobile.Target = new DeleteItemByLayerTarget( Layer.FacialHair );
		}

		private class KillResTarget : Target
		{
			private bool m_Kill;

			public KillResTarget( bool kill ) : base( -1, false, TargetFlags.None )
			{
				m_Kill = kill;
			}

			protected override void OnTarget( Mobile from, object targeted )
			{
				if ( targeted is Mobile )
				{
					if ( m_Kill )
						((Mobile)targeted).Kill();
					else
						((Mobile)targeted).Resurrect();
				}
				else
				{
					from.SendMessage( "Target a mobile." );
				}
			}
		}

		private static void Kill_OnCommand( CommandEventArgs e )
		{
			e.Mobile.Target = new KillResTarget( true );
		}

		private static void Resurrect_OnCommand( CommandEventArgs e )
		{
			e.Mobile.Target = new KillResTarget( false );
		}

		private class NewXYZTarget : Target
		{
			private int m_X, m_Y, m_Z;
			private bool m_xSet, m_ySet, m_zSet;

			public NewXYZTarget( int x, bool xSet, int y, bool ySet, int z, bool zSet ) : base( -1, false, TargetFlags.None )
			{
				m_X = x; m_xSet = xSet;
				m_Y = y; m_ySet = ySet;
				m_Z = z; m_zSet = zSet;
			}

			protected override void OnTarget( Mobile from, object targeted )
			{
				if ( targeted is Item )
				{
					Item item = (Item)targeted;

					item.Location = new Point3D( m_xSet ? m_X : item.X, m_ySet ? m_Y : item.Y, m_zSet ? m_Z : item.Z );
				}
				else if ( targeted is Mobile )
				{
					Mobile m = (Mobile)targeted;

					m.Location = new Point3D( m_xSet ? m_X : m.X, m_ySet ? m_Y : m.Y, m_zSet ? m_Z : m.Z );
				}
			}
		}

		private static void NewXYZ_OnCommand( CommandEventArgs e )
		{
			if ( e.Length == 3 )
			{
				e.Mobile.Target = new NewXYZTarget( e.GetInt32( 0 ), true, e.GetInt32( 1 ), true, e.GetInt32( 2 ), true );
			}
			else
			{
				e.Mobile.SendMessage( "Format: NewXYZ <x> <y> <z>" );
			}
		}

		private static void NewX_OnCommand( CommandEventArgs e )
		{
			if ( e.Length == 1 )
			{
				e.Mobile.Target = new NewXYZTarget( e.GetInt32( 0 ), true, 0, false, 0, false );
			}
			else
			{
				e.Mobile.SendMessage( "Format: NewX <x>" );
			}
		}

		private static void NewY_OnCommand( CommandEventArgs e )
		{
			if ( e.Length == 1 )
			{
				e.Mobile.Target = new NewXYZTarget( 0, false, e.GetInt32( 0 ), true, 0, false );
			}
			else
			{
				e.Mobile.SendMessage( "Format: NewY <y>" );
			}
		}

		private static void NewZ_OnCommand( CommandEventArgs e )
		{
			if ( e.Length == 1 )
			{
				e.Mobile.Target = new NewXYZTarget( 0, false, 0, false, e.GetInt32( 0 ), true );
			}
			else
			{
				e.Mobile.SendMessage( "Format: NewZ <z>" );
			}
		}

		private class IncXYZTarget : Target
		{
			private int m_xOffset, m_yOffset, m_zOffset;

			public IncXYZTarget( int xOffset, int yOffset, int zOffset ) : base( -1, false, TargetFlags.None )
			{
				m_xOffset = xOffset;
				m_yOffset = yOffset;
				m_zOffset = zOffset;
			}

			protected override void OnTarget( Mobile from, object targeted )
			{
				if ( targeted is Item )
				{
					Item item = (Item)targeted;

					item.Location = new Point3D( item.X + m_xOffset, item.Y + m_yOffset, item.Z + m_zOffset );
				}
				else if ( targeted is Mobile )
				{
					Mobile m = (Mobile)targeted;

					m.Location = new Point3D( m.X + m_xOffset, m.Y + m_yOffset, m.Z + m_zOffset );
				}
			}
		}

		private static void IncXYZ_OnCommand( CommandEventArgs e )
		{
			if ( e.Length == 3 )
			{
				e.Mobile.Target = new IncXYZTarget( e.GetInt32( 0 ), e.GetInt32( 1 ), e.GetInt32( 2 ) );
			}
			else
			{
				e.Mobile.SendMessage( "Format: IncXYZ <x> <y> <z>" );
			}
		}

		private static void IncX_OnCommand( CommandEventArgs e )
		{
			if ( e.Length == 1 )
			{
				e.Mobile.Target = new IncXYZTarget( e.GetInt32( 0 ), 0, 0 );
			}
			else
			{
				e.Mobile.SendMessage( "Format: IncX <x>" );
			}
		}

		private static void IncY_OnCommand( CommandEventArgs e )
		{
			if ( e.Length == 1 )
			{
				e.Mobile.Target = new IncXYZTarget( 0, e.GetInt32( 0 ), 0 );
			}
			else
			{
				e.Mobile.SendMessage( "Format: IncY <y>" );
			}
		}

		private static void IncZ_OnCommand( CommandEventArgs e )
		{
			if ( e.Length == 1 )
			{
				e.Mobile.Target = new IncXYZTarget( 0, 0, e.GetInt32( 0 ) );
			}
			else
			{
				e.Mobile.SendMessage( "Format: IncZ <z>" );
			}
		}

		private static void APN_OnCommand( CommandEventArgs e )
		{
			Mobile m = e.Mobile;

			m.AutoPageNotify = !m.AutoPageNotify;

			m.SendMessage( "Your auto-page-notify has been turned {0}.", m.AutoPageNotify ? "on" : "off" );
		}

		private static void Animate_OnCommand( CommandEventArgs e )
		{
			if ( e.Length == 6 )
			{
				e.Mobile.Animate( e.GetInt32( 0 ), e.GetInt32( 1 ), e.GetInt32( 2 ), e.GetBoolean( 3 ), e.GetBoolean( 4 ), e.GetInt32( 5 ) );
			}
			else
			{
				e.Mobile.SendMessage( "Format: Animate <action> <frameCount> <repeatCount> <forward> <repeat> <delay>" );
			}
		}

		private class TeleportTarget : Target
		{
			public TeleportTarget() : base( -1, true, TargetFlags.None )
			{
			}

			protected override void OnTarget( Mobile from, object targeted )
			{
				IPoint3D p = targeted as IPoint3D;

				if ( p != null )
				{
					if ( p is Item )
						p = ((Item)p).GetWorldLocation();

					Effects.SendLocationEffect( from.Location, from.Map, 0x3728, 13 );

					from.Location = new Point3D( p );

					Effects.PlaySound( p, from.Map, 0x1FE );
					Effects.SendLocationEffect( from.Location, from.Map, 0x3728, 13 );
				}
			}
		}

		private static void Teleport_OnCommand( CommandEventArgs e )
		{
			e.Mobile.Target = new TeleportTarget();
		}

		public class SetHiddenTarget : Target
		{
			private bool m_Value;

			public SetHiddenTarget( bool value ) : base( 18, false, TargetFlags.None )
			{
				m_Value = value;
			}

			protected override void OnTarget( Mobile from, object targeted )
			{
				if ( targeted is Mobile )
				{
					Mobile m = (Mobile)targeted;

					Effects.SendLocationEffect( new Point3D( m.X + 1, m.Y, m.Z + 4 ), m.Map, 0x3728, 13 );
					Effects.SendLocationEffect( new Point3D( m.X + 1, m.Y, m.Z ), m.Map, 0x3728, 13 );
					Effects.SendLocationEffect( new Point3D( m.X + 1, m.Y, m.Z - 4 ), m.Map, 0x3728, 13 );
					Effects.SendLocationEffect( new Point3D( m.X, m.Y + 1, m.Z + 4 ), m.Map, 0x3728, 13 );
					Effects.SendLocationEffect( new Point3D( m.X, m.Y + 1, m.Z ), m.Map, 0x3728, 13 );
					Effects.SendLocationEffect( new Point3D( m.X, m.Y + 1, m.Z - 4 ), m.Map, 0x3728, 13 );

					Effects.SendLocationEffect( new Point3D( m.X + 1, m.Y + 1, m.Z + 11 ), m.Map, 0x3728, 13 );
					Effects.SendLocationEffect( new Point3D( m.X + 1, m.Y + 1, m.Z + 7 ), m.Map, 0x3728, 13 );
					Effects.SendLocationEffect( new Point3D( m.X + 1, m.Y + 1, m.Z + 3 ), m.Map, 0x3728, 13 );
					Effects.SendLocationEffect( new Point3D( m.X + 1, m.Y + 1, m.Z - 1 ), m.Map, 0x3728, 13 );

					m.PlaySound( 0x228 );
					m.Hidden = m_Value;
				}
			}
		}

		private static void Hide_OnCommand( CommandEventArgs e )
		{
			e.Mobile.Target = new SetHiddenTarget( true );
		}

		private static void Unhide_OnCommand( CommandEventArgs e )
		{
			e.Mobile.Target = new SetHiddenTarget( false );
		}

		private static void Cast_OnCommand( CommandEventArgs e )
		{
			if ( e.Length == 1 )
			{
				Spell spell = SpellRegistry.NewSpell( e.GetString( 0 ), e.Mobile, null );

				if ( spell != null )
				{
					spell.Cast();
				}
				else
				{
					e.Mobile.SendMessage( "That spell was not found." );
				}
			}
			else
			{
				e.Mobile.SendMessage( "Format: Cast <name>" );
			}
		}

		private class StuckMenuTarget : Target
		{
			public StuckMenuTarget() : base( -1, false, TargetFlags.None )
			{
			}

			protected override void OnTarget( Mobile from, object targeted )
			{
				if ( targeted is Mobile )
				{
					from.SendMenu( new StuckMenu( from, (Mobile) targeted, false ) );
				}
			}
		}

		private static void Stuck_OnCommand( CommandEventArgs e )
		{
			e.Mobile.Target = new StuckMenuTarget();
		}
	}
}