using System;
using Server.Items;
using Server.Network;

namespace Server.Scripts.Items.Weapons.Ranged
{
	public class BaseRanged : BaseWeapon
	{
		private int m_EffectID;

		public BaseRanged( int effectID, int minDamage, int maxDamage, int speed, int hitSound, int missSound, int maxRange, SkillName skill, WeaponType type, WeaponAnimation anim, int itemID, Layer layer ) : base( minDamage, maxDamage, speed, hitSound, missSound, maxRange, skill, type, anim, itemID, layer )
		{
			m_EffectID = effectID;
		}

		public BaseRanged( Serial serial ) : base( serial )
		{
		}

		public override TimeSpan OnSwing( Mobile attacker, Mobile defender )
		{
			// Make sure we've been standing still for one second
			if ( DateTime.Now > (attacker.LastMoveTime + TimeSpan.FromSeconds( 1.0 )) )
			{
				if ( attacker.HarmfulCheck( defender ) )
				{
					if ( OnFired( attacker, defender ) )
					{
						if ( attacker.CheckMeleeHit( this, defender ) )
							OnHit( attacker, defender );
						else
							OnMiss( attacker, defender );
					}
				}

				return GetDelay( attacker );
			}
			else
			{
				return TimeSpan.FromSeconds( 0.25 );
			}
		}

		public override void OnHit( Mobile attacker, Mobile defender )
		{
			base.OnHit( attacker, defender );
		}

		public override void OnMiss( Mobile attacker, Mobile defender )
		{
			base.OnMiss( attacker, defender );
		}

		public virtual bool OnFired( Mobile attacker, Mobile defender )
		{
			Effects.SendMovingEffect( attacker, defender, m_EffectID, 7, 1, false, false );

			return true;
		}

		public override void Serialize( GenericWriter writer )
		{
			base.Serialize( writer );

			writer.Write( (int) 0 ); // version

			writer.Write( (int) m_EffectID );
		}

		public override void Deserialize( GenericReader reader )
		{
			base.Deserialize( reader );

			int version = reader.ReadInt();

			switch ( version )
			{
				case 0:
				{
					m_EffectID = reader.ReadInt();
					break;
				}
			}
		}
	}
}