using System;
using Server.Items;
using Server.Scripts.Items;
using Server.Mobiles;
using Server.Scripts.Mobiles.Animals.Mounts;

namespace Server.Scripts.Misc
{
	public class CharacterCreation
	{
		public static void Initialize()
		{
			// Register our event handler
			EventSink.CharacterCreated += new CharacterCreatedEventHandler( EventSink_CharacterCreated );
		}

		private static void FillBackpack( Mobile m )
		{
			Container pack = m.Backpack;

			if ( pack != null )
			{
				pack.DropItem( MakeNewbie( new Gold( 1000 ) ) ); // Start gold can be customized here
				pack.DropItem( MakeNewbie( new Server.Scripts.Items.Weapons.Knives.Dagger() ) ); // Everyone starts with a dagger
			}
		}

		private static Item MakeNewbie( Item item )
		{
			item.Newbied = true;

			return item;
		}

		private static void FillBankbox( Mobile m )
		{
			BankBox bank = m.BankBox;

			if ( bank != null )
			{
				bank.DropItem( new BankCheck( 1000000 ) );

				// Full spellbook
				Spellbook book = new Spellbook();

				book.Content = ulong.MaxValue;

				bank.DropItem( book );

				// Bag containing 50 of each reagent
				bank.DropItem( new BagOfReagents( 50 ) );

				// Add tools here as they are implemented
				bank.DropItem( new Scissors() );

				// A few dye tubs
				bank.DropItem( new Dyes() );
				bank.DropItem( new DyeTub() );
				bank.DropItem( new DyeTub() );
				bank.DropItem( new BlackDyeTub() );

				DyeTub darkRedTub = new DyeTub();

				darkRedTub.DyedHue = 0x485;
				darkRedTub.Redyable = false;

				bank.DropItem( darkRedTub );

				// Some food
				bank.DropItem( new Apple( 1000 ) );

				// Resources
				bank.DropItem( new Feather( 1000 ) );
				bank.DropItem( new BoltOfCloth( 1000 ) );
				bank.DropItem( new BlankScroll( 1000 ) );
				bank.DropItem( new Hides( 1000 ) );
				bank.DropItem( new Bandage( 1000 ) );
				bank.DropItem( new Bottle( 1000 ) );
				bank.DropItem( new Log( 1000 ) );

				bank.DropItem( new IronIngot( 5000 ) );
				bank.DropItem( new DullCopperIngot( 5000 ) );
				bank.DropItem( new ShadowIronIngot( 5000 ) );
				bank.DropItem( new CopperIngot( 5000 ) );
				bank.DropItem( new BronzeIngot( 5000 ) );
				bank.DropItem( new GoldIngot( 5000 ) );
				bank.DropItem( new AgapiteIngot( 5000 ) );
				bank.DropItem( new VeriteIngot( 5000 ) );
				bank.DropItem( new ValoriteIngot( 5000 ) );

				// Reagents
				bank.DropItem( new BlackPearl( 1000 ) );
				bank.DropItem( new Bloodmoss( 1000 ) );
				bank.DropItem( new Garlic( 1000 ) );
				bank.DropItem( new Ginseng( 1000 ) );
				bank.DropItem( new MandrakeRoot( 1000 ) );
				bank.DropItem( new Nightshade( 1000 ) );
				bank.DropItem( new SulfurousAsh( 1000 ) );
				bank.DropItem( new SpidersSilk( 1000 ) );

				// Some extra starting gold
				bank.DropItem( new Gold( 9000 ) );

				// 5 blank recall runes
				for ( int i = 0; i < 5; ++i )
				{
					bank.DropItem( new RecallRune() );
				}
			}
		}

		private static void EventSink_CharacterCreated( CharacterCreatedEventArgs args )
		{
			Mobile newChar = args.Mobile;

			FillBackpack( newChar );
			FillBankbox( newChar );

			newChar.AddItem( MakeNewbie( new Server.Scripts.Items.Weapons.Swords.Katana() ) ); // Give them a katana

			BaseMount mount;

			switch ( Utility.Random( 4 ) )
			{default:
				case 0: mount = new TanHorse(); break;
				case 1: mount = new GrayHorse(); break;
				case 2: mount = new BrownHorse(); break;
				case 3: mount = new DirtyHorse(); break;
			}

			mount.Rider = newChar; // And have them ride it

			// Start the welcome timer
			WelcomeTimer timer = new WelcomeTimer( newChar );
			timer.Start();
		}
	}
}