using System;
using System.Text;
using Server.Gumps;
using Server.Network;

namespace Server.Scripts.Misc.TestCenter
{
	public class TCHelpGump : Gump
	{
		public static void Initialize()
		{
			// Register our speech handler
			EventSink.Speech += new SpeechEventHandler( EventSink_Speech );
		}

		private static void EventSink_Speech( SpeechEventArgs args )
		{
			if ( !args.Handled && args.Speech.ToUpper() == "HELP" )
			{
				args.Mobile.SendGump( new TCHelpGump() );

				args.Handled = true;
			}
		}

		public TCHelpGump() : base( 40, 40 )
		{
			AddPage( 0 );
			AddBackground( 0, 0, 160, 120, 5054 );

			AddButton( 10, 10, 0xFB7, 0xFB9, 1, GumpButtonType.Reply, 0 );
			AddLabel( 45, 10, 0x34, "RunUO.com" );

			AddButton( 10, 35, 0xFB7, 0xFB9, 2, GumpButtonType.Reply, 0 );
			AddLabel( 45, 35, 0x34, "List of skills" );

			AddButton( 10, 60, 0xFB7, 0xFB9, 3, GumpButtonType.Reply, 0 );
			AddLabel( 45, 60, 0x34, "Command list" );

			AddButton( 10, 85, 0xFB1, 0xFB3, 0, GumpButtonType.Reply, 0 );
			AddLabel( 45, 85, 0x34, "Close" );
		}

		public override void OnResponse( NetState sender, RelayInfo info )
		{
			switch ( info.ButtonID )
			{
				case 1: // RunUO.com
				{
					sender.LaunchBrowser( "http://www.RunUO.com" );
					break;
				}
				case 2: // List of skills
				{
					string[] strings = Enum.GetNames( typeof( SkillName ) );

					Array.Sort( strings );

					StringBuilder sb = new StringBuilder();

					if ( strings.Length > 0 )
						sb.Append( strings[0] );

					for ( int i = 0; i < strings.Length; ++i )
					{
						string v = strings[i];

						if ( (sb.Length + 1 + v.Length) >= 256 )
						{
							sender.Send( new AsciiMessage( Server.Serial.MinusOne, -1, MessageType.Label, 0x35, 3, "System", sb.ToString() ) );
							sb = new StringBuilder();
							sb.Append( v );
						}
						else
						{
							sb.Append( ' ' );
							sb.Append( v );
						}
					}

					if ( sb.Length > 0 )
					{
						sender.Send( new AsciiMessage( Server.Serial.MinusOne, -1, MessageType.Label, 0x35, 3, "System", sb.ToString() ) );
					}

					break;
				}
				case 3: // Command list
				{
					string[] commands = new string[]
						{
							"The command prefix is [",
							"[add <typeName> [params]",
							"[tile <typeName> [params]",
							"[delete",
							"[props",
							"[set <name> <value>",
							"[get <name>",
							"[go [<x> <y> <z>]",
							"[pages",
							"[stuckmenu",
							"[hide [unhide",
							"[shavehair [shavebeard",
							"[incxyz [incx [incy [incz",
							"[newxyz [newx [newy [newz",
						};

					for ( int i = 0; i < commands.Length; ++i )
					{
						sender.Send( new AsciiMessage( Server.Serial.MinusOne, -1, MessageType.Label, 0x35, 3, "System", commands[i] ) );
					}

					break;
				}
			}
		}
	}
}