using System;
using System.Collections;
using Server.Spells;
using Server.Spells.First;
using Server.Spells.Second;
using Server.Spells.Third;
using Server.Spells.Fourth;
using Server.Spells.Fifth;
using Server.Spells.Sixth;
using Server.Spells.Seventh;
using Server.Targeting;

namespace Server.Scripts.Mobiles.Monsters.Magical
{
	public abstract class BaseAggressive : Mobile
	{
		private Timer m_Timer;

		public BaseAggressive()
		{
			m_Timer = new AITimer( this );
			m_Timer.Start();
		}

		public BaseAggressive( Serial serial ) : base( serial )
		{
			m_Timer = new AITimer( this );
			m_Timer.Start();
		}

		public override void Serialize( GenericWriter writer )
		{
			base.Serialize( writer );

			writer.Write( (int) 0 ); // version
		}

		public override void Deserialize( GenericReader reader )
		{
			base.Deserialize( reader );

			int version = reader.ReadInt();
		}

		public virtual double GetValueFrom( Mobile m )
		{
			return (m.Skills[SkillName.Magery].Value + m.Int) / this.GetDistanceToSqrt( m );
		}

		public override void OnAfterDelete()
		{
			m_Timer.Stop();

			base.OnAfterDelete();
		}

		public virtual void AquireCombatant()
		{
			Mobile comb = null;
			double val = double.MinValue;

			IPooledEnumerable eable = this.Map.GetMobilesInRange( this.Location, 6 );

			foreach ( Mobile m in eable )
			{
				if ( m != this && this.CanSee( m ) )
				{
					double theirVal = GetValueFrom( m );

					if ( theirVal > val )
					{
						comb = m;
						val = theirVal;
					}
				}
			}

			eable.Free();

			this.Combatant = comb;
		}

		protected override void OnTargetChange()
		{
			Target t = this.Target;

			if ( t != null )
			{
				if ( (t.Flags & TargetFlags.Harmful) != 0 && this.Combatant != null )
				{
					if ( this.Target.Range == -1 || this.InRange( this.Combatant, t.Range ) )
					{
						t.Invoke( this, this.Combatant );
					}
				}
				else if ( (t.Flags & TargetFlags.Beneficial) != 0 )
				{
					t.Invoke( this, this );
				}
				else
				{
					t.Cancel( this, TargetCancelType.Canceled );
				}
			}
		}

		private class AITimer : Timer
		{
			private BaseAggressive m_Owner;
			private DateTime m_NextCastTime;
			private int m_Combo = -1;

			public AITimer( BaseAggressive owner ) : base( TimeSpan.FromSeconds( 1.0 ), TimeSpan.FromSeconds( 1.0 ) )
			{
				this.Priority = TimerPriority.FiftyMS;

				m_Owner = owner;
			}

			protected override void OnTick()
			{
				if ( m_Owner.Map == Map.Internal )
				{
					Stop();
					return;
				}

				if ( m_Owner.Combatant == null )
				{
					m_Owner.AquireCombatant();
				}
				else
				{
					Mobile c = m_Owner.Combatant;

					if ( m_Owner.Target != null )
					{
						if ( m_Owner.Combatant != null )
						{
							if ( m_Owner.Target.Range == -1 || m_Owner.InRange( m_Owner.Combatant, m_Owner.Target.Range ) )
							{
								m_Owner.Target.Invoke( m_Owner, m_Owner.Combatant );
							}
							else if ( !m_Owner.InRange( c, 1 ) )
							{
								m_Owner.Move( m_Owner.GetDirectionTo( c ) | Direction.Running );
							}
						}
						else
						{
							m_Owner.Target.Cancel( m_Owner, TargetCancelType.Canceled );
						}
					}
					else if ( DateTime.Now > m_NextCastTime && m_Owner.Spell == null && m_Owner.InRange( c, 8 ) )
					{
						Spell spell = null;

						if ( m_Owner.Poisoned )
						{
							spell = new CureSpell( m_Owner, null );
						}
						else if ( m_Combo != -1 )
						{
							if ( m_Combo == 0 )
							{
								spell = new WeakenSpell( m_Owner, null );
								++m_Combo;
							}
							else if ( m_Combo == 1 )
							{
								if ( !c.Poisoned )
									spell = new PoisonSpell( m_Owner, null );

								++m_Combo;
							}

							if ( m_Combo == 2 && spell == null )
							{
								switch ( Utility.Random( 3 ) )
								{
									default:
									case 0:
									{
										if ( c.Int < c.Dex )
											spell = new FeeblemindSpell( m_Owner, null );
										else
											spell = new ClumsySpell( m_Owner, null );

										++m_Combo;

										break;
									}
									case 1:
									{
										spell = new EnergyBoltSpell( m_Owner, null );
										m_Combo = -1;
										break;
									}
									case 2:
									{
										spell = new FlameStrikeSpell( m_Owner, null );
										m_Combo = -1;
										break;
									}
								}
							}
							else if ( m_Combo == 3 && spell == null )
							{
								spell = new MindBlastSpell( m_Owner, null );
								m_Combo = -1;
							}
						}
						else
						{
							if ( (c.Spell is HealSpell || c.Spell is GreaterHealSpell) && !c.Poisoned )
							{
								spell = new PoisonSpell( m_Owner, null );
							}
							else
							{
								switch ( Utility.Random( 4 ) )
								{
									default:
									case 0:
									{
										if ( m_Owner.Hits < (m_Owner.HitsMax - 50) )
											spell = new GreaterHealSpell( m_Owner, null );
										else if ( m_Owner.Hits < (m_Owner.HitsMax - 20) )
											spell = new HealSpell( m_Owner, null );

										break;
									}
									case 1:
									{
										if ( !c.Poisoned )
											spell = new PoisonSpell( m_Owner, null );

										break;
									}
									case 2:
									{
										switch ( Utility.Random( 8 ) )
										{
											default:
											case 0: spell = new MagicArrowSpell( m_Owner, null ); break;
											case 1: spell = new HarmSpell( m_Owner, null ); break;
											case 2: spell = new FireballSpell( m_Owner, null ); break;
											case 3: spell = new LightningSpell( m_Owner, null ); break;
											case 4: spell = new MindBlastSpell( m_Owner, null ); break;
											case 5: spell = new EnergyBoltSpell( m_Owner, null ); break;
											case 6: spell = new ExplosionSpell( m_Owner, null ); break;
											case 7: spell = new FlameStrikeSpell( m_Owner, null ); break;
										}

										break;
									}
									case 3:
									{
										if ( m_Owner.Mana > 60 )
										{
											m_Combo = 0;
											spell = new ExplosionSpell( m_Owner, null );
										}

										break;
									}
								}
							}
						}

						if ( spell != null && spell.Cast() )
						{
							m_NextCastTime = DateTime.Now + TimeSpan.FromSeconds( 0.18 );
						}
						else if ( !m_Owner.InRange( c, 1 ) )
						{
							m_Owner.Move( m_Owner.GetDirectionTo( c ) | Direction.Running );
						}
					}
					else if ( !m_Owner.InRange( c, 1 ) )
					{
						m_Owner.Move( m_Owner.GetDirectionTo( c ) | Direction.Running );
					}
				}

				if ( m_Owner.Combatant == null )
				{
					this.Interval = TimeSpan.FromSeconds( 5.0 );
					this.Priority = TimerPriority.OneSecond;
				}
				else
				{
					this.Interval = TimeSpan.FromSeconds( 0.2 );
					this.Priority = TimerPriority.TwentyFiveMS;
				}
			}
		}
	}
}