using System;
using Server.Targeting;
using Server.Network;

namespace Server.Spells.First
{
	public class FeeblemindSpell : Spell
	{
		private static SpellInfo m_Info = new SpellInfo(
				"Feeblemind", "Rel Wis",
				SpellCircle.First,
				Reagent.Ginseng,
				Reagent.Nightshade
			);

		public FeeblemindSpell( Mobile caster, Server.Items.SpellScroll scroll ) : base( caster, scroll, m_Info )
		{
		}

		public override void OnCast()
		{
			Caster.Target = new InternalTarget( this );
		}

		public void Target( Mobile m )
		{
			if ( !Caster.CanSee( m ) )
			{
				Caster.SendLocalizedMessage( 500237 ); // Target can not be seen.
			}
			else if ( Caster.HarmfulCheck( m ) && CheckSequence() )
			{
				int decrease = 1 + (int)(Caster.Skills[SkillName.Magery].Value * 0.1);
				TimeSpan duration = TimeSpan.FromSeconds( Caster.Skills[SkillName.Magery].Value * 1.2 );

				m.AddStatMod( new StatMod( StatType.Int, "[Spell] Int Offset", -decrease, duration ) );

				if ( m.Spell != null && m.Spell.State == SpellState.Casting )
				{
					m.Spell.Disturb();
				}

				m.Paralyzed = false;

				if ( Caster != m )
				{
					Caster.Direction = Caster.GetDirectionTo( m );
				}

				m.PlaySound( 0x1E4 );
				Effects.SendTargetEffect( m, 14201, 16 );
			}

			FinishSequence();
		}

		private class InternalTarget : Target
		{
			private FeeblemindSpell m_Owner;

			public InternalTarget( FeeblemindSpell owner ) : base( 12, false, TargetFlags.Harmful )
			{
				m_Owner = owner;
			}

			protected override void OnTarget( Mobile from, object o )
			{
				if ( o is Mobile )
				{
					m_Owner.Target( (Mobile)o );
				}
			}

			protected override void OnTargetFinish( Mobile from )
			{
				m_Owner.FinishSequence();
			}
		}
	}
}