using System;
using Server.Targeting;
using Server.Network;

namespace Server.Spells.Seventh
{
	public class EnergyFieldSpell : Spell
	{
		private static SpellInfo m_Info = new SpellInfo(
				"Energy Field", "In Sanct Grav",
				SpellCircle.Seventh,
				Reagent.Bloodmoss,
				Reagent.MandrakeRoot,
				Reagent.SpidersSilk,
				Reagent.SulfurousAsh
			);

		public EnergyFieldSpell( Mobile caster, Server.Items.SpellScroll scroll ) : base( caster, scroll, m_Info )
		{
		}

		public override void OnCast()
		{
			Caster.Target = new InternalTarget( this );
		}

		public void Target( IPoint3D p )
		{
			if ( !Caster.CanSee( p ) )
			{
				Caster.SendLocalizedMessage( 500237 ); // Target can not be seen.
			}
			else if ( CheckSequence() )
			{
				int dx = Caster.Location.X - p.X;
				int dy = Caster.Location.Y - p.Y;
				int rx = (dx - dy) * 44;
				int ry = (dx + dy) * 44;

				bool eastToWest;

				if ( rx >= 0 && ry >= 0 )
				{
					eastToWest = false;
				}
				else if ( rx >= 0 )
				{
					eastToWest = true;
				}
				else if ( ry >= 0 )
				{
					eastToWest = true;
				}
				else
				{
					eastToWest = false;
				}

				if ( Caster != p )
				{
					Caster.Direction = Caster.GetDirectionTo( p );
				}

				Effects.PlaySound( p, Caster.Map, 0x20B );

				TimeSpan duration = TimeSpan.FromSeconds( Caster.Skills[SkillName.Magery].Value * 0.28 + 2.0 ); // (28% of magery) + 2.0 seconds
				int itemID = eastToWest ? 0x3947 : 0x3956;

				for ( int i = -2; i <= 2; ++i )
				{
					Item item = new Item( itemID );

					item.Movable = false;

					item.Location = new Point3D( eastToWest ? p.X + i : p.X, eastToWest ? p.Y : p.Y + i, p.Z );
					item.Map = Caster.Map;

					InternalTimer t = new InternalTimer( item, duration );
					t.Start();
				}
			}

			FinishSequence();
		}

		private class InternalTimer : Timer
		{
			private Item m_Item;

			public InternalTimer( Item item, TimeSpan duration ) : base( duration )
			{
				m_Item = item;
			}

			protected override void OnTick()
			{
				m_Item.Delete();
			}
		}

		private class InternalTarget : Target
		{
			private EnergyFieldSpell m_Owner;

			public InternalTarget( EnergyFieldSpell owner ) : base( 12, true, TargetFlags.None )
			{
				m_Owner = owner;
			}

			protected override void OnTarget( Mobile from, object o )
			{
				if ( o is IPoint3D )
				{
					m_Owner.Target( (IPoint3D)o );
				}
			}

			protected override void OnTargetFinish( Mobile from )
			{
				m_Owner.FinishSequence();
			}
		}
	}
}