using System;
using Server.Targeting;
using Server.Network;

namespace Server.Spells.Third
{
	public class TeleportSpell : Spell
	{
		private static SpellInfo m_Info = new SpellInfo(
				"Teleport", "Rel Por",
				SpellCircle.Third,
				Reagent.Bloodmoss,
				Reagent.SulfurousAsh
			);

		public TeleportSpell( Mobile caster, Server.Items.SpellScroll scroll ) : base( caster, scroll, m_Info )
		{
		}

		public override void OnCast()
		{
			Caster.Target = new InternalTarget( this );
		}

		public void Target( IPoint3D p )
		{
			if ( !Caster.CanSee( p ) )
			{
				Caster.SendLocalizedMessage( 500237 ); // Target can not be seen.
			}
			else if ( CheckSequence() )
			{
				Mobile m = Caster;

				Effects.SendLocationEffect( m.Location, m.Map, 0x3728, 13 );

				if ( m != p )
				{
					m.Direction = m.GetDirectionTo( p );
				}

				m.Location = new Point3D( p );

				Effects.SendLocationEffect( m.Location, m.Map, 0x3728, 13 );

				m.PlaySound( 0x1FE );
			}

			FinishSequence();
		}

		private class InternalTarget : Target
		{
			private TeleportSpell m_Owner;

			public InternalTarget( TeleportSpell owner ) : base( 12, true, TargetFlags.None )
			{
				m_Owner = owner;
			}

			protected override void OnTarget( Mobile from, object o )
			{
				IPoint3D p = o as IPoint3D;

				if ( p != null )
				{
					m_Owner.Target( p );
				}
			}

			protected override void OnTargetFinish( Mobile from )
			{
				m_Owner.FinishSequence();
			}
		}
	}
}